/* eslint camelcase:0 */
import fetch from 'cross-fetch'
import Fellow from 'fellow'
import { getReposFromUsers, getReposFromSearch } from 'getrepos'
import { githubAuthorizationHeader } from 'githubauthreq'
const ghapi = process.env.GITHUB_API || 'https://api.github.com'
/** Export the Fellow class we have imported and are using, such that consumers of this package and ensure they are interacting with the same singletons */
export { Fellow }
/** Continue despite certain errors */
function ignore() {
	// console.warn(err)
	return new Set()
}
/** Fetch the full profile information for a contributor */
export async function getContributorProfile(url) {
	const resp = await fetch(url, {
		headers: {
			Authorization: githubAuthorizationHeader(),
			Accept: 'application/vnd.github.v3+json',
		},
	})
	const responseData = await resp.json()
	// Check
	if (responseData.message) {
		return Promise.reject(new Error(responseData.message))
	}
	// Return
	return responseData
}
/** Fetch contributors from a Repository's GitHub Contributor API */
export async function getContributorsFromCommits(slug) {
	// Fetch
	const url = `${ghapi}/repos/${slug}/contributors?per_page=100`
	const resp = await fetch(url, {
		headers: {
			Authorization: githubAuthorizationHeader(),
			Accept: 'application/vnd.github.v3+json',
		},
	})
	const responseData = await resp.json()
	// Check
	if (responseData.message) {
		return Promise.reject(new Error(responseData.message))
	} else if (!Array.isArray(responseData)) {
		return Promise.reject(
			new Error('response was not an array of contributors')
		)
	} else if (responseData.length === 0) {
		return new Set()
	}
	// Process
	return new Set(
		await Promise.all(
			responseData.map(async function (contributor) {
				const profile = await getContributorProfile(contributor.url)
				const fellow = Fellow.ensure({
					githubProfile: profile,
					name: profile.name,
					email: profile.email,
					description: profile.bio,
					company: profile.company,
					location: profile.location,
					homepage: profile.blog,
					hireable: profile.hireable,
					githubUsername: profile.login,
					githubUrl: profile.html_url,
				})
				fellow.contributions.set(slug, contributor.contributions)
				if (contributor.site_admin) {
					fellow.administeredRepositories.add(slug)
				}
				fellow.contributedRepositories.add(slug)
				return fellow
			})
		)
	)
}
/** Fetch contributors from a repository's `package.json` file */
export async function getContributorsFromPackage(slug) {
	// Fetch
	const url = `http://raw.github.com/${slug}/master/package.json`
	const resp = await fetch(url)
	const packageData = await resp.json()
	// Process
	const added = new Set()
	if (packageData.author) {
		Fellow.add(packageData.author).forEach((fellow) => {
			fellow.authoredRepositories.add(slug)
			added.add(fellow)
		})
	}
	for (const contributor of packageData.contributors || []) {
		const fellow = Fellow.ensure(contributor)
		fellow.contributedRepositories.add(slug)
		added.add(fellow)
	}
	for (const maintainer of packageData.maintainers || []) {
		const fellow = Fellow.ensure(maintainer)
		fellow.contributedRepositories.add(slug)
		added.add(fellow)
	}
	// Return
	return added
}
/** Fetch contributors from a GitHub repository slug */
export async function getContributorsFromRepo(slug) {
	return Fellow.flatten(
		await Promise.all([
			getContributorsFromCommits(slug),
			getContributorsFromPackage(slug).catch(ignore),
		])
	)
}
/** Fetch contributors from GitHub repository slugs (e.g. `bevry/getcontributors`) */
export async function getContributorsFromRepos(slugs) {
	return Fellow.flatten(
		await Promise.all(slugs.map((slug) => getContributorsFromRepo(slug)))
	)
}
/** Fetch contributors for all repositories, within the GitHub organisations and usernames (e.g. `bevry`, `balupton`) */
export async function getContributorsFromOrgs(orgs) {
	const repos = await getReposFromUsers(orgs)
	// Filter out forks and grab the slugs
	const slugs = repos
		.filter((repo) => repo.fork !== true)
		.map((repo) => repo.full_name)
	// Fetch the contributors for the repos
	return getContributorsFromRepos(slugs)
}
/** Fetch contributors for all repositories that match a certain search query */
export async function getContributorsFromSearch(query, opts) {
	const repos = await getReposFromSearch(query, opts)
	// Just grab the slugs
	const slugs = repos.map((repo) => repo.full_name)
	// Fetch the contributors for the repos
	return getContributorsFromRepos(slugs)
}
