'use strict'
/* eslint camelcase:0 */
var __importDefault =
	(this && this.__importDefault) ||
	function (mod) {
		return mod && mod.__esModule ? mod : { default: mod }
	}
Object.defineProperty(exports, '__esModule', { value: true })
const cross_fetch_1 = __importDefault(require('cross-fetch'))
const fellow_1 = __importDefault(require('fellow'))
exports.Fellow = fellow_1.default
const getrepos_1 = require('getrepos')
const githubauthreq_1 = require('githubauthreq')
const ghapi = process.env.GITHUB_API || 'https://api.github.com'
/** Continue despite certain errors */
function ignore() {
	// console.warn(err)
	return new Set()
}
/** Fetch the full profile information for a contributor */
async function getContributorProfile(url) {
	const resp = await cross_fetch_1.default(url, {
		headers: {
			Authorization: githubauthreq_1.githubAuthorizationHeader(),
			Accept: 'application/vnd.github.v3+json',
		},
	})
	const responseData = await resp.json()
	// Check
	if (responseData.message) {
		return Promise.reject(new Error(responseData.message))
	}
	// Return
	return responseData
}
exports.getContributorProfile = getContributorProfile
/** Fetch contributors from a Repository's GitHub Contributor API */
async function getContributorsFromCommits(slug) {
	// Fetch
	const url = `${ghapi}/repos/${slug}/contributors?per_page=100`
	const resp = await cross_fetch_1.default(url, {
		headers: {
			Authorization: githubauthreq_1.githubAuthorizationHeader(),
			Accept: 'application/vnd.github.v3+json',
		},
	})
	const responseData = await resp.json()
	// Check
	if (responseData.message) {
		return Promise.reject(new Error(responseData.message))
	} else if (!Array.isArray(responseData)) {
		return Promise.reject(
			new Error('response was not an array of contributors')
		)
	} else if (responseData.length === 0) {
		return new Set()
	}
	// Process
	return new Set(
		await Promise.all(
			responseData.map(async function (contributor) {
				const profile = await getContributorProfile(contributor.url)
				const fellow = fellow_1.default.ensure({
					githubProfile: profile,
					name: profile.name,
					email: profile.email,
					description: profile.bio,
					company: profile.company,
					location: profile.location,
					homepage: profile.blog,
					hireable: profile.hireable,
					githubUsername: profile.login,
					githubUrl: profile.html_url,
				})
				fellow.contributions.set(slug, contributor.contributions)
				if (contributor.site_admin) {
					fellow.administeredRepositories.add(slug)
				}
				fellow.contributedRepositories.add(slug)
				return fellow
			})
		)
	)
}
exports.getContributorsFromCommits = getContributorsFromCommits
/** Fetch contributors from a repository's `package.json` file */
async function getContributorsFromPackage(slug) {
	// Fetch
	const url = `http://raw.github.com/${slug}/master/package.json`
	const resp = await cross_fetch_1.default(url)
	const packageData = await resp.json()
	// Process
	const added = new Set()
	if (packageData.author) {
		fellow_1.default.add(packageData.author).forEach((fellow) => {
			fellow.authoredRepositories.add(slug)
			added.add(fellow)
		})
	}
	for (const contributor of packageData.contributors || []) {
		const fellow = fellow_1.default.ensure(contributor)
		fellow.contributedRepositories.add(slug)
		added.add(fellow)
	}
	for (const maintainer of packageData.maintainers || []) {
		const fellow = fellow_1.default.ensure(maintainer)
		fellow.contributedRepositories.add(slug)
		added.add(fellow)
	}
	// Return
	return added
}
exports.getContributorsFromPackage = getContributorsFromPackage
/** Fetch contributors from a GitHub repository slug */
async function getContributorsFromRepo(slug) {
	return fellow_1.default.flatten(
		await Promise.all([
			getContributorsFromCommits(slug),
			getContributorsFromPackage(slug).catch(ignore),
		])
	)
}
exports.getContributorsFromRepo = getContributorsFromRepo
/** Fetch contributors from GitHub repository slugs (e.g. `bevry/getcontributors`) */
async function getContributorsFromRepos(slugs) {
	return fellow_1.default.flatten(
		await Promise.all(slugs.map((slug) => getContributorsFromRepo(slug)))
	)
}
exports.getContributorsFromRepos = getContributorsFromRepos
/** Fetch contributors for all repositories, within the GitHub organisations and usernames (e.g. `bevry`, `balupton`) */
async function getContributorsFromOrgs(orgs) {
	const repos = await getrepos_1.getReposFromUsers(orgs)
	// Filter out forks and grab the slugs
	const slugs = repos
		.filter((repo) => repo.fork !== true)
		.map((repo) => repo.full_name)
	// Fetch the contributors for the repos
	return getContributorsFromRepos(slugs)
}
exports.getContributorsFromOrgs = getContributorsFromOrgs
/** Fetch contributors for all repositories that match a certain search query */
async function getContributorsFromSearch(query, opts) {
	const repos = await getrepos_1.getReposFromSearch(query, opts)
	// Just grab the slugs
	const slugs = repos.map((repo) => repo.full_name)
	// Fetch the contributors for the repos
	return getContributorsFromRepos(slugs)
}
exports.getContributorsFromSearch = getContributorsFromSearch
